searchData={"items":[{"type":"task","doc":"A task to clean any stale lockfiles and blend build artifacts under the\n`blend/` folder.","title":"mix blend.clean","ref":"Mix.Tasks.Blend.Clean.html"},{"type":"task","doc":"A task to resolve your blends and generate lockfiles.\n\nThat is to say, have blend read your defined blends in `blend.exs` and generate\na lockfile for each blend by combining each specific blend overrides with your package\ndependencies in `mix.exs`.","title":"mix blend.get","ref":"Mix.Tasks.Blend.Get.html"},{"type":"task","doc":"A task to generates an empty blend.exs file for you\nto define your blends, if you don't yet have one.\n\nIt is created with a commented example in it.\n\n```\n$ mix blend.init\nSuccessfully created blend.exs file\n```","title":"mix blend.init","ref":"Mix.Tasks.Blend.Init.html"},{"type":"task","doc":"A task to list to quickly get a list of blend names you have defined in\nyour blend.exs file.\n\n```\n$ mix blend.list\nplug_crypto_1\nplug_crypto_2\n```","title":"mix blend.list","ref":"Mix.Tasks.Blend.List.html"},{"type":"task","doc":"Task to generate the `premix.exs` file, which helps confuguring your\npackage project to easily run any mix task against a specific blend\nlockfile.","title":"mix blend.premix","ref":"Mix.Tasks.Blend.Premix.html"},{"type":"task","doc":"A task to update your blend lockfiles dependencies.\n\nThat means, trying to get all the dependencies in each lockfile to their most recent\nversion possible respecting the defined version constraints.","title":"mix blend.update","ref":"Mix.Tasks.Blend.Update.html"},{"type":"extras","doc":"# 🥣 Blend\n\n[![ci](https://github.com/mimiquate/blend/actions/workflows/ci.yml/badge.svg?branch=main)](https://github.com/mimiquate/blend/actions?query=branch%3Amain)\n[![Hex.pm](https://img.shields.io/hexpm/v/blend.svg)](https://hex.pm/packages/blend)\n[![Docs](https://img.shields.io/badge/docs-gray.svg)](https://hexdocs.pm/blend)\n\nTest your package against different versions of its dependencies.\n\nGenerates and maintains multiple lockfiles based on your defined variations (a.k.a. blends)\nso that you can test your package against different variations of your dependencies versions.\n\nYou can read more about the motivation for this project [here](#Motivation).","title":"🥣 Blend","ref":"readme.html"},{"type":"extras","doc":"If [available in Hex](https://hex.pm/docs/publish), the package can be installed\nby adding `blend` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:blend, \"~> 0.4.0\", only: :dev}\n  ]\nend\n```","title":"Installation - 🥣 Blend","ref":"readme.html#installation"},{"type":"extras","doc":"","title":"Usage - 🥣 Blend","ref":"readme.html#usage"},{"type":"extras","doc":"```\nmix blend.init\n```","title":"1. Generate `blend.exs` - 🥣 Blend","ref":"readme.html#1-generate-blend-exs"},{"type":"extras","doc":"Edit and set your blends in the `blend.exs` file.\n\nFor example, an elixir package that depends on `plug_crypto` with a requirement of `~> 1.2 or ~> 2.0`,\nthat wants to test against the two major versions, would normally have it's `mix.lock` resolved to\n`2.x`, so it would want to define the following:\n\n```elixir\n# blend.exs\n\n# Example for testing against plug_crypto 1.x\n%{\n  plug_crypto_1: [{:plug_crypto, \"~> 1.2\"}]\n}\n```\n\nin order for blend to generate an additional lockfile that locks `plug_crypto` to the latest possible\n`1.x` supported version.\n\nMap keys define the blend name, used for naming the lockfile, and the dependencies list are\nmerged with the package dependencies before resolving and generating the lockfile variation.","title":"2. Define your blends - 🥣 Blend","ref":"readme.html#2-define-your-blends"},{"type":"extras","doc":"```\nmix blend.get\n```\n\nto resolve your blends and generate new lockfiles with variations of your dependencies under the new `/blend` folder.\n\n```\nblend\n├── _build\n├── deps\n├── plug_crypto_1.mix.lock\nmix.lock\n```","title":"3. Resolve blends and generate lockfiles - 🥣 Blend","ref":"readme.html#3-resolve-blends-and-generate-lockfiles"},{"type":"extras","doc":"```diff\n # .gitignore\n\n ...\n\n+/blend/_build\n+/blend/deps\n```\n\nAdd to your `.gitignore` file, before comitting your changes.","title":"4. Ignore blend build artifacts - 🥣 Blend","ref":"readme.html#4-ignore-blend-build-artifacts"},{"type":"extras","doc":"#### Option A. Overriding your `mix.lock`.\n\nIf you just need to quickly run against a blend lockfile (e.g. in a CI job step), it might be enough to just:\n\n```\ncp blend/plug_crypto_1.mix.lock mix.lock\n```\n\nbefore your `mix deps.get` step.\n\nAfter that you can run any additional task, e.g. run your tests or your phx.server, which will run in the context\nof the specific blend lockfile copied.\n\n```sh\nmix test\n```\n\nor\n\n```sh\nmix phx.server\n```\n\n#### Option B. `BLEND` env var configuration\n\nA more permanent configuration for running mix tasks in the context of a blend lockfile with a simple env var\ncan be accomplished by customizing your `mix.exs` a bit, with the following steps.\n\n##### 1. Create a new file `blend/premix.exs` with the following command:\n\n```\nmix blend.premix\n```\n\nThis will generate a `blend/premix.exs` file that needs to be compiled at the top of your `mix.exs` file\nso that some mix env vars are properly set based on the `BLEND` env var before running any mix task.\n\n##### 2. Modify your `mix.exs`.\n\n```diff\n # mix.exs\n\n+if File.exists?(\"blend/premix.exs\") do\n+  Code.compile_file(\"blend/premix.exs\")\n+end\n\n defmodule YourApp.MixProject do\n   ...\n   def project do\n     [\n       ...\n     ]\n+    |> Keyword.merge(maybe_lockfile_option())\n   end\n   ....\n\n+  defp maybe_lockfile_option do\n+    case System.get_env(\"MIX_LOCKFILE\") do\n+      nil -> []\n+      \"\" -> []\n+      lockfile -> [lockfile: lockfile]\n+     end\n+  end\n end\n```\n\n##### 3. Enjoy\n\nNow you can run any task, e.g. run your tests or your server, against different lockfiles locally by just\nprepending the proper environment variable value:\n\n```sh\nBLEND=plug_crypto_1 mix test\n```\n\nor\n\n```sh\nBLEND=plug_crypto_1 mix phx.server\n```","title":"5. Running tasks in the context of a blend lockfile - 🥣 Blend","ref":"readme.html#5-running-tasks-in-the-context-of-a-blend-lockfile"},{"type":"extras","doc":"```\nmix blend.init         # Generate blend.exs\nmix blend.get          # Generate blend lockfiles\nmix blend.update --all # Update blend lockfiles to latest possible versions\nmix blend.list         # List blends\nmix blend.clean        # Cleans blends build artifacts and stale lockfiles\nmix blend.premix       # Generate premix.exs file\n```","title":"Tasks - 🥣 Blend","ref":"readme.html#tasks"},{"type":"extras","doc":"At Mimiquate, our engagement with the Elixir ecosystem deeply influences our projects. We’ve benefited immensely from the community’s support and resources. During our development of open-source hex packages, we encountered a challenge: testing across different dependency versions lacked a straightforward approach. This issue led us to think about how we could contribute to the community that has been so supportive of us.\n\nOur background includes Ruby on Rails, where the Ruby community has tackled a similar challenge with the appraisal gem. This tool makes it easier to test various dependency versions and is designed to work well with CI pipelines, reducing the risk of regressions. It’s been really great for developers and has positively impacted the ecosystem’s health.\n\nWith that inspiration, we set out to create something for the Elixir community to help streamline development. Our first version doesn’t have all the functionalities of the appraisal gem but establishes a good starting point. It allows developers to test their hex packages against different lock file combinations, which is handy for spotting regressions in common scenarios.\n\nWe realize that our current tool doesn’t address every testing scenario. Yet, we’re committed to improving it, aiming to cover more ground and enhance its functionality. Our goal goes beyond just solving a technical issue; we hope to make the development process in the Elixir community more supportive and efficient for everyone.","title":"Motivation - 🥣 Blend","ref":"readme.html#motivation"},{"type":"extras","doc":"At Mimiquate we have been Elixiring since 2016 and we are true believers in the community and the ecosystem.\nIf you are looking to turbo charge your Elixir team, [reach out](mailto:contact@mimiquate.com?subject=Elixir%20team%20augmentation)!","title":"Having a hard time finding Elixir talent? - 🥣 Blend","ref":"readme.html#having-a-hard-time-finding-elixir-talent"},{"type":"extras","doc":"Copyright 2024 Mimiquate\n\nLicensed under the Apache License, Version 2.0 (the \"License\");\nyou may not use this file except in compliance with the License.\nYou may obtain a copy of the License at\n\n    http://www.apache.org/licenses/LICENSE-2.0\n\nUnless required by applicable law or agreed to in writing, software\ndistributed under the License is distributed on an \"AS IS\" BASIS,\nWITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.\nSee the License for the specific language governing permissions and\nlimitations under the License.","title":"License - 🥣 Blend","ref":"readme.html#license"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,50]}}